#ifndef NPSTAT_MEMOIZINGSYMBETAFILTERPROVIDER_HH_
#define NPSTAT_MEMOIZINGSYMBETAFILTERPROVIDER_HH_

/*!
// \file MemoizingSymbetaFilterProvider.hh
//
// \brief Builds symmetric beta LOrPE filters and remembers
//        these filters when the user sets the corresponding flag
//
// Author: I. Volobouev
//
// November 2013
*/

#include <map>
#include <vector>
#include <string>
#include <cassert>

#include "geners/ClassId.hh"

#include "npstat/stat/AbsSymbetaFilterProvider.hh"
#include "npstat/stat/AllSymbetaParams1D.hh"

namespace npstat {
    class MemoizingSymbetaFilterProvider : public SimpleSymbetaFilterProvider
    {
    public:
        inline MemoizingSymbetaFilterProvider() {this->startMemoizing();}

        inline virtual ~MemoizingSymbetaFilterProvider() {}

        virtual CPP11_shared_ptr<const LocalPolyFilter1D> provideFilter(
            int symbetaPower, double bandwidth, double degree,
            unsigned nbins, double binwidth,
            const BoundaryHandling& bm,
            unsigned excludedBin, bool excludeCentralPoint);

        inline unsigned long nMemoized() const {return filterMap_.size();}

        void firstMemoizedInfo(int* symbetaPower, double* bandwidth,
                               double* degree, unsigned* nbins,
                               double* binwidth, BoundaryHandling* bm,
                               unsigned* excludedBin,
                               bool* excludeCentralPoint) const;

        std::vector<double> knownBandwidthValues() const;

        bool operator==(const MemoizingSymbetaFilterProvider& r) const;
        inline bool operator!=(const MemoizingSymbetaFilterProvider& r) const
            {return !(*this == r);}

        // I/O methods needed for writing
        inline gs::ClassId classId() const {return gs::ClassId(*this);}
        bool write(std::ostream& of) const;

        // I/O methods needed for reading
        static inline const char* classname()
            {return "npstat::MemoizingSymbetaFilterProvider";}
        static inline unsigned version() {return 3;}
        static void restore(const gs::ClassId& id, std::istream& in,
                            MemoizingSymbetaFilterProvider* ptr);
    private:
        typedef CPP11_shared_ptr<const LocalPolyFilter1D> FilterPtr;
        typedef std::map<AllSymbetaParams1D, FilterPtr> FilterMap;

        FilterMap filterMap_;
    };
}

#endif // NPSTAT_MEMOIZINGSYMBETAFILTERPROVIDER_HH_
