#ifndef NPSTAT_DOUBLEHISTOBINS1D_HH_
#define NPSTAT_DOUBLEHISTOBINS1D_HH_

#include <stdexcept>

#include "npstat/wrap/NumpyTypecode.hh"
#include "npstat/stat/HistoND.hh"

namespace npstat {
    template <class Histo>
    PyObject* doubleHistoBins1D(const Histo& h, const bool doubleSize=false,
                                const bool mirror=true)
    {
        typedef typename Histo::value_type Numeric;

        if (h.dim() != 1) throw std::invalid_argument(
            "In npstat::doubleHistoBins1D: 1-d input histogram expected");
        const unsigned nBins = h.nBins();
        const unsigned arrSize = doubleSize ? 2U*nBins : nBins;
        const int typenum = NumpyTypecode<double>::code;
        npy_intp sh = arrSize;
        PyObject* arr = PyArray_SimpleNew(1, &sh, typenum);
        if (arr)
        {
            const Numeric* from = h.binContents().data();
            double* a = (double*)PyArray_DATA((PyArrayObject*)arr);
            for (unsigned i=0; i<nBins; ++i)
                a[i] = static_cast<double>(from[i]);

            if (doubleSize)
            {
                if (mirror)
                    for (unsigned i=0; i<nBins; ++i)
                        a[i+nBins] = static_cast<double>(from[nBins-1-i]);
                else
                    for (unsigned i=nBins; i<arrSize; ++i)
                        a[i] = 0.0;
            }
        }
        return arr;
    }
}

#endif // NPSTAT_DOUBLEHISTOBINS1D_HH_
